/*==============================================================================
*
*  LOGIC CORE:          PS2 键盘解码实验			
*  MODULE NAME:         Beep()
*  COMPANY:             北京至芯开源科技有限责任公司
*                       www.zxopen.com   www.fpgaw.com
*	author:					小梅哥
*	author QQ：528369266
*  REVISION HISTORY:  
*
*    Revision 1.0  25/12/2014     Description: Initial Release.
*
*  FUNCTIONAL DESCRIPTION:
*
*  本模块为PS2键盘解码实验中，PS2解码模块
*
*	每来一个时钟周期的en高电平信号，蜂鸣器响200ms
* 
===============================================================================*/

module Beep(Clk,Rst_n,En,Beep);

//---------模块输入端口------------	
	input Clk;
	input Rst_n;
	input En;
	
//---------模块输出端口------------	
	output reg Beep;

//----------用户寄存器-------------		
	reg [25:0]counter;
	reg [1:0]state;
	
//--------内部连接信号线------------
	wire beep_en;
	
//----------模块内部常量定义--------	
	localparam CNT_TOP = 5_000_000;//used in physical board Clk = 50M  

//-------定义蜂鸣器工作状态----------
	localparam IDEL = 2'b01;
	localparam BUSY = 2'b10;

//-----产生蜂鸣器鸣叫条件------------	
always@(posedge Clk or negedge Rst_n)
begin
	if(!Rst_n)
		begin
			state <= IDEL;
			counter <= 0;
		end
	else
		case(state)
			IDEL:	
				if(En)
					state<=BUSY;
				else
					state<=IDEL;
			
			BUSY:
				if(counter<CNT_TOP)
				begin
					counter<=counter+1'b1;
					state<=BUSY;
				end	
				else 
				begin
					counter<=0;
					state<=IDEL;
				end
		endcase	
end

//---------根据蜂鸣器鸣叫条件控制蜂鸣器-------
always @ (posedge Clk or negedge Rst_n)
begin
	if(!Rst_n)
		Beep <= 1'b1;
	else 
	begin
		if(beep_en)
			Beep <= 1'b1;
		else
			Beep <= 1'b0;
	end
end

assign beep_en = (counter[14] == 1)? 1'b1:1'b0;

endmodule
