/******************************************************************************
*
*  LOGIC CORE:          PS2 键盘解码实验			
*  MODULE NAME:         DIG_LED_DRIVE()
*  COMPANY:             北京至芯开源科技有限责任公司
*                       www.zxopen.com   www.fpgaw.com
*	author:					小梅哥
*	author QQ：528369266
*  REVISION HISTORY:  
*
*    Revision 1.0  25/12/2014     Description: Initial Release.
*
*  FUNCTIONAL DESCRIPTION:
*
*  本模块为PS2键盘解码实验中，数码管驱动模块
******************************************************************************/

/*数码管扫描模块,位选为外部74hc138译码器进行控制*/
/*仿真时请将本文件设置为顶层，并在代码中根据相应注释中的内容选择cnt1_MAX = 24*/

module DIG_LED_DRIVE(Data,Clk,Rst_n,Dig_Led_seg,Dig_Led_sel);

	input [23:0]Data;
	input Clk;
	input Rst_n;
	output [7:0]Dig_Led_seg;
	output [2:0]Dig_Led_sel;
	
	parameter system_clk = 50_000_000;
	
//	localparam cnt1_MAX = 24;/*仿真的时候使用，板级验证时请注释掉*/
	localparam cnt1_MAX = system_clk/1000/2-1;/*板级验证的时候使用，仿真时请注释掉*/

	reg [14 :0] cnt1;
	reg clk_1K;//扫描时钟，1KHz
	reg [2:0]sel_r;//数码管位选
	reg [7:0]seg_r;//数码管段选
	reg [3:0]disp_data;//单位显示数据缓存

	//1KHz时钟分频计数器
	always@(posedge Clk)
	begin
		if(!Rst_n)cnt1<=0;
		else if(cnt1==cnt1_MAX)cnt1<=0;
		else cnt1<=cnt1+1'b1;
	end

	//得到1KHz时钟
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)clk_1K<=0;
	else if(cnt1==cnt1_MAX)
		clk_1K<=~clk_1K;
	else ;

	//位选信号控制
	always@(posedge clk_1K or negedge Rst_n)
	if(!Rst_n)sel_r<=3'd0;
	else if(sel_r == 3'd5)
		sel_r<=3'd0;
	else
		sel_r<=sel_r+1'b1;

	//根据不同的数码管位选择不同的待显示数据
	always@(*)
	if(!Rst_n)
		disp_data<=4'd0;
	else
	begin
		case(sel_r)
			0:disp_data<=Data[23:20];
			1:disp_data<=Data[19:16];
			2:disp_data<=Data[15:12];
			3:disp_data<=Data[11:8];
			4:disp_data<=Data[7:4];
			5:disp_data<=Data[3:0];
			default :disp_data<=4'd0;
		endcase
	end

	//数据译码，将待显示数据翻译为符合数码管显示的编码
	always@(*)
	if(!Rst_n)
		seg_r<=8'hff;
	else
	begin
		case(disp_data)
			4'd0:		seg_r<=8'hc0;
			4'd1:		seg_r<=8'hf9;
			4'd2:		seg_r<=8'ha4;
			4'd3:		seg_r<=8'hb0;
			4'd4:		seg_r<=8'h99;
			4'd5:		seg_r<=8'h92;
			4'd6:		seg_r<=8'h82;
			4'd7:		seg_r<=8'hf8;
			4'd8:		seg_r<=8'h80;
			4'd9:		seg_r<=8'h90;
			4'd10:	seg_r<=8'h88;
			4'd11:	seg_r<=8'h83;
			4'd12:	seg_r<=8'hc6;
			4'd13:	seg_r<=8'ha1;
			4'd14:	seg_r<=8'h86;
			4'd15:	seg_r<=8'h8e;
			default : seg_r<=8'hff;
		endcase
	end

	assign Dig_Led_seg = seg_r;
	assign Dig_Led_sel = sel_r;

endmodule
