/*=============================================================================
*
*  LOGIC CORE:          PS2 键盘解码实验			
*  MODULE NAME:         PS2_Key_Board_Driver_tb()
*  COMPANY:             北京至芯开源科技有限责任公司
*                       www.zxopen.com   www.fpgaw.com
*	author:					小梅哥
*	author QQ：528369266
*  REVISION HISTORY:  
*
*    Revision 1.0  25/12/2014     Description: Initial Release.
*
*  FUNCTIONAL DESCRIPTION:
*
*  本模块为PS2键盘解码实验中，顶层模块的testbench
*
===============================================================================*/

`timescale 1ns/1ns

module Key_Disp_tb;

	reg Clk;/*system clock*/
	reg Rst_n;/*复位信号*/
	
	reg PS2_Din;/*PS2键盘数据线*/
	reg PS2_Clk;/*PS2键盘时钟线*/
	
	
	wire [7:0]Dig_Led_seg;/*数码管段选输出*/
	wire [2:0]Dig_Led_sel;/*数码管位选输出*/
	
	wire Beep;/*蜂鸣器输出脚*/

	Key_Disp Key_Disp(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.PS2_Din(PS2_Din),
		.PS2_Clk(PS2_Clk),
		.Dig_Led_seg(Dig_Led_seg),
		.Dig_Led_sel(Dig_Led_sel),
		.Beep(Beep)
	);
	
	initial begin
		Clk = 1;
		Rst_n = 0;
		PS2_Din = 1;
		PS2_Clk = 1;
		#200;
		Rst_n = 1;
		Key_Event(8'h1A);	/* Z */
		#40000;
		Key_Event(8'h35);	/* X */
		#80000;
		Key_Event(8'h44);	/* O */
		#13200;
		Key_Event(8'h4D);	/* P */
		#25600;
		Key_Event(8'h24);	/* E */
		#12300;
		Key_Event(8'h31);	/* N */
		#20000;
		Long_Key_Event(8'h70);	/* "INSERT" */
		#40000;
		Long_Key_Event(8'h6c);	/* "HOME" */
		#80000;
		Long_Key_Event(8'h7d);	/* "PAGE UP" */
		#13200;
		Long_Key_Event(8'h71);	/* "DELETE" */
		#25600;
		Long_Key_Event(8'h69);	/* "END" */
		#12300;
		Long_Key_Event(8'h7a);	/* "PAGE DOWN" */
		#2000000;
		$stop;
	end	
	
/*---------生成工作时钟-----------*/
	always #10 Clk = ~Clk;
	
/*----任务：以PS2协议发送一个字节的数据-----*/
	task Send_data;
		input [7:0]Data;
		begin
			PS2_Din = 0;	/*发送起始位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[0];/*发送第0位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[1];/*发送第1位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[2];/*发送第2位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[3];/*发送第3位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[4];/*发送第4位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[5];/*发送第5位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[6];/*发送第6位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = Data[7];/*发送第7位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = 0;/*暂时忽略校验位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;
			
			#20000;PS2_Din = 1;/*停止位*/
			#20000;PS2_Clk = 0;
			#40000;PS2_Clk = 1;			
		end
	endtask

/*-----任务：模拟按下按键的操作------*/	
	task press_key;
		input [7:0]Key_Number;
		begin
			Send_data(Key_Number);
			#50000;		
		end
	endtask
	
/*-----任务：模拟释放按键的操作------*/		
	task release_key;
		input [7:0]Key_Number;
		begin
			Send_data(8'hF0);
			#50000;
			Send_data(Key_Number);
			#50000;
		end
	endtask

/*----任务：模拟一次短码的按下和释放操作-----*/	
	task Key_Event;
		input [7:0]Key_Number;
		begin
			press_key(Key_Number);
			#30000;
			release_key(Key_Number);
		end
	endtask

/*----任务：模拟一次长码的按下和释放操作-----*/		
	task Long_Key_Event;
		input [7:0]Key_Number;
		begin
			press_key(8'he0);
			#30000;
			press_key(Key_Number);
			#30000;
			press_key(8'he0);
			#30000;
			release_key(Key_Number);
		end
	endtask
	
endmodule
