/*============================================================================
*
*  LOGIC CORE:          PS2 键盘解码实验			
*  MODULE NAME:         PS2_Key_Board_Driver()
*  COMPANY:             北京至芯开源科技有限责任公司
*                       www.zxopen.com   www.fpgaw.com
*	author:					小梅哥
*	author QQ：528369266
*  REVISION HISTORY:  
*
*    Revision 1.0  25/12/2014     Description: Initial Release.
*
*  FUNCTIONAL DESCRIPTION:
*
*  本模块为PS2键盘解码实验中，PS2解码模块
===========================================================================*/

module PS2_Key_Board_Driver(Clk,Rst_n,PS2_Din,PS2_Clk,Key_Flag,Key_Valve);

	input Clk;	/*system clock*/
	input Rst_n;	/*复位信号*/
	
	input PS2_Din;	/*PS2键盘数据线*/
	input PS2_Clk;	/*PS2键盘时钟线*/
	
	output reg Key_Flag;	/*解码得到键值标志信号*/
	output reg [9:0] Key_Valve;	/*解码结果，其中最高位为通/断码识别位，0为通码，1为断码，
										次高位为长短码识别位，0为短码，1为长码，低八位为码值*/

	reg [3:0]Cnt1;
	reg Break_r;	/*断码（按键释放释放）标志信号*/
	reg Long_Code_r;	/*长码标志信号*/
	reg [7:0]Data_tmp;	/*解码数据缓存*/
	reg nedge_PS2_Clk_Shift;	/*1bit移位寄存器，通过移位将PS2的下降沿检测标志滞后一拍*/
	
	reg PS2_Clk_Tmp0,PS2_Clk_Tmp1,PS2_Clk_Tmp2,PS2_Clk_Tmp3;
	
	wire nedge_PS2_Clk;	/*PS2从机时钟下降沿检测标志信号*/
	
	always @ (posedge Clk or negedge Rst_n)
	if(!Rst_n) begin
		PS2_Clk_Tmp0 <= 1'b0;
		PS2_Clk_Tmp1 <= 1'b0;
		PS2_Clk_Tmp2 <= 1'b0;	
		PS2_Clk_Tmp3 <= 1'b0;
	end
	else begin
		PS2_Clk_Tmp0 <= PS2_Clk;
		PS2_Clk_Tmp1 <= PS2_Clk_Tmp0;
		PS2_Clk_Tmp2 <= PS2_Clk_Tmp1;
		PS2_Clk_Tmp3 <= PS2_Clk_Tmp2;			
	end

	/*-------获取PS时钟信号的下降沿-------------*/	
	assign nedge_PS2_Clk = !PS2_Clk_Tmp0 & !PS2_Clk_Tmp1 & PS2_Clk_Tmp2 & PS2_Clk_Tmp3;
	
	always @(posedge Clk)nedge_PS2_Clk_Shift <= nedge_PS2_Clk;

/*------------PS2时钟下降沿个数计数器-----------------------*/	
	always @(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		Cnt1 <= 4'd0;
	else if(Cnt1 == 4'd11)
		Cnt1 <= 4'd0;
	else if(nedge_PS2_Clk)
		Cnt1 <= Cnt1 + 1'b1;

/*--------------读取8位数据位---------------*/		
	always @ (posedge Clk or negedge Rst_n)
	if(!Rst_n)
		Data_tmp <= 8'd0;
	else if(nedge_PS2_Clk_Shift) begin
		case(Cnt1)
			4'd2:Data_tmp[0] <= PS2_Din;
			4'd3:Data_tmp[1] <= PS2_Din;
			4'd4:Data_tmp[2] <= PS2_Din;
			4'd5:Data_tmp[3] <= PS2_Din;
			4'd6:Data_tmp[4] <= PS2_Din;
			4'd7:Data_tmp[5] <= PS2_Din;
			4'd8:Data_tmp[6] <= PS2_Din;
			4'd9:Data_tmp[7] <= PS2_Din;
			default:Data_tmp <= Data_tmp;
		endcase	
	end
	else
		Data_tmp <= Data_tmp;

	always @ (posedge Clk or negedge Rst_n)
	if(!Rst_n) begin
		Break_r <= 1'b0;
		Key_Valve <= 10'd0;
		Key_Flag <= 1'b0;
		Long_Code_r <= 1'b0;
	end
	else if(Cnt1 == 4'd11) begin
		if(Data_tmp == 8'hE0)	/*判断是否为长码*/
			Long_Code_r <= 1'b1;		/*将长码标志置1*/
		else if(Data_tmp == 8'hF0)		/*判断是否为断码*/
			Break_r <= 1'b1;		/*将断码标志置1*/
		else begin	/*检测到的数据为通码*/
			Key_Valve <= {Break_r,Long_Code_r,Data_tmp};/*将长码标志、断码标志和解码到的按键码输出*/
			Key_Flag <= 1'b1;	/*产生解码成功标志信号*/
			Long_Code_r <= 1'b0;	/*清零长码标志*/
			Break_r <= 1'b0;	/*清零断码标志*/		
		end		
	end
	else begin
		Key_Valve <= Key_Valve;
		Key_Flag <= 1'b0;
		Break_r <= Break_r;
		Long_Code_r <= Long_Code_r;
	end

endmodule
